------------------------------------------------------
-- AdSpace.lua
-- Version 1700.1
------------------------------------------------------

FAS_SPECIAL_VENDOR_COLOR = {r=1.0, g=0.5, b=0.25};
FAS_INDENT = "  ";

-- Configuration
FAS_Config = { };
FAS_Config.Tooltip = true;
FAS_Config.ShowCost = false;

local function FAS_Tooltip_Hook(frame, name, link, count, data)
	FAS_Old_Tooltip_Hook(frame, name, link, count, data);
	if ( FAS_Config.Tooltip ) then
		local _, _, itemID  = string.find(link, ".Hitem:(%d+):%d+:%d+:%d+.h%[[^]]+%].h");
		itemID = tonumber(itemID);
		local itemInfo = FAS_ItemInfo[itemID];

		if (itemInfo ~= nil) then
			local myFaction = UnitFactionGroup("player");
			local vendors = GFWTable.Merge(FAS_VendorInfo[myFaction][itemID], FAS_VendorInfo["Neutral"][itemID]);
			local note = itemInfo.note;
			
			local color;
			local intro;
			if (note ~= nil) then
				color = FAS_SPECIAL_VENDOR_COLOR;
			else
				color = GFW_FONT_COLOR;
			end
			if (FAS_Config.ShowCost) then
				intro = string.format(SOLD_FOR_PRICE_BY, FAS_GetTextGSC(itemInfo.b));
			else
				intro = SOLD_BY;
			end
					
			frame.eDone = 1;
			if (table.getn(vendors) > 1) then
				frame:AddLine(intro..":", color.r, color.g, color.b);
				for i, aVendor in vendors do
					local vendorLoc = FAS_VendorLocations[aVendor];
					if (vendorLoc ~= nil) then
						frame:AddLine(FAS_INDENT..string.format(VENDOR_LOCATION_FORMAT, aVendor, vendorLoc), color.r, color.g, color.b);
					else
						GFWUtils.Print(GFWUtils.Red("AdSpace error: ").."Can't find location for "..aVendor..". Please report to gazmik@fizzwidget.com.");
					end
				end
			elseif (table.getn(vendors) == 1) then
				local vendorLoc = FAS_VendorLocations[vendors[1]];
				if (vendorLoc ~= nil) then
					frame:AddLine(intro.." "..string.format(VENDOR_LOCATION_FORMAT, vendors[1], vendorLoc), color.r, color.g, color.b);
				else
					GFWUtils.Print(GFWUtils.Red("AdSpace error: ").."Can't find location for "..aVendor..". Please report to gazmik@fizzwidget.com.");
				end
			else
				local found = false;
				for _, faction in {"Alliance", "Horde", "Neutral"} do
					if (FAS_VendorInfo[faction][itemID] ~= nil) then
						found = true;
					end
				end
				if not (found) then
					GFWUtils.Print(GFWUtils.Red("AdSpace error: ")..link.."("..itemID..") is listed but has no vendors. Please report to gazmik@fizzwidget.com.");
				end
			end
			if (note ~= "") then
				frame:AddLine(note, color.r, color.g, color.b);
			end
			frame:Show();
			return;
		end
		
		local libramInfo = FAS_LibramInfo[itemID];
		if (libramInfo ~= nil) then
			local color = FAS_SPECIAL_VENDOR_COLOR;
			frame.eDone = 1;
			frame:AddLine(RETURN_TO.." "..string.format(VENDOR_LOCATION_FORMAT, libramInfo.name, FAS_VendorLocations[libramInfo.name]), color.r, color.g, color.b);
			frame:AddLine(string.format(ARCANUM_FORMAT, libramInfo.bonus), color.r, color.g, color.b);
			frame:Show();
			return;
		end

		local otherInfo = FAS_OtherItemInfo[itemID];
		if (otherInfo ~= nil) then
			local color = FAS_SPECIAL_VENDOR_COLOR;
			frame.eDone = 1;
			frame:AddLine(otherInfo, color.r, color.g, color.b);
			frame:Show();
			return;
		end
	end
	
end

function FAS_OnLoad()

	-- Register Slash Commands
	SLASH_FAS1 = "/adspace";
	SLASH_FAS2 = "/ads";
	SlashCmdList["FAS"] = function(msg)
		FAS_ChatCommandHandler(msg);
	end
	
	-- Hook in new tooltip code
	FAS_Old_Tooltip_Hook = TT_AddTooltip;
	TT_AddTooltip = FAS_Tooltip_Hook;

	GFWUtils.Print("Fizzwidget AdSpace initialized!");
end

function FAS_ChatCommandHandler(msg)

	-- Print Help
	if ( msg == "help" ) or ( msg == "" ) then
		GFWUtils.Print("Fizzwidget AdSpace:");
		GFWUtils.Print("/adspace (or /ads)");
		GFWUtils.Print("- "..GFWUtils.Hilite("help").." - Print this helplist.");
		GFWUtils.Print("- "..GFWUtils.Hilite("status").." - Check current settings.");
		GFWUtils.Print("- "..GFWUtils.Hilite("tooltip on").." | "..GFWUtils.Hilite("off").." - Enable/disable display of vendor availability in recipe tooltips.");
		GFWUtils.Print("- "..GFWUtils.Hilite("showcost on").." | "..GFWUtils.Hilite("off").." - Also show vendor prices for recipes.");
		GFWUtils.Print("- "..GFWUtils.Hilite("info <item link>").." - Show vendor info for an item in the chat window.");
		return;
	end
	
	if (msg == "tooltip on") then
		FAS_Config.Tooltip = true;
		if ( FAS_Config.ShowCost ) then
			GFWUtils.Print("Showing vendor availability (or related info) and pricing in tooltips for select items.");
		else
			GFWUtils.Print("Only showing vendor availability (or related info) in tooltips for select items.");
		end
		return;
	end
	if (msg == "tooltip off") then
		FAS_Config.Tooltip = false;
		GFWUtils.Print(GFWUtils.Hilite("Not").." showing extra info in item tooltips.");
		return;
	end
	
	if (msg == "showcost on") then
		FAS_Config.ShowCost = true;
		GFWUtils.Print("Showing vendor availability (or related info) and pricing in tooltips for select items.");
		if not ( FAS_Config.Tooltip ) then
			GFWUtils.Print("(or would, if tooltip modification was enabled. Type "..GFWUtils.Hilite("/ads tooltip on").." to do so.)");
		end
		return;
	end
	if (msg == "showcost off") then
		FAS_Config.ShowCost = false;
		if ( FAS_Config.Tooltip ) then
			GFWUtils.Print("Only showing vendor availability (or related info) in tooltips for select items.");
		else
			GFWUtils.Print(GFWUtils.Hilite("Not").." showing extra info in item tooltips.");
		end
		return;
	end
	
	if ( msg == "status" ) then
		if ( FAS_Config.Tooltip ) then
			if ( FAS_Config.ShowCost ) then
				GFWUtils.Print("Showing vendor availability (or related info) and pricing in tooltips for select items.");
			else
				GFWUtils.Print("Only showing vendor availability (or related info) in tooltips for select items.");
			end
		else
			GFWUtils.Print(GFWUtils.Hilite("Not").." showing extra info in item tooltips.");
		end
		return;
	end
	
	if (msg == "test") then
		local itemInfoCount = 0;
		for itemID in FAS_ItemInfo do
			local found = false;
			for _, faction in {"Alliance", "Horde", "Neutral"} do
				if (FAS_VendorInfo[faction][itemID] ~= nil) then
					found = true;
				end
			end
			if not (found) then
				GFWUtils.Print("Item ID "..itemID.." not found in FAS_VendorInfo.");
			end
			itemInfoCount = itemInfoCount + 1;
		end
		GFWUtils.Print(itemInfoCount.." entries in FAS_ItemInfo.");
		for _, faction in {"Alliance", "Horde", "Neutral"} do
			local vendorInfoCount = 0;
			for itemID in FAS_VendorInfo[faction] do
				if (FAS_ItemInfo[itemID] == nil) then
					GFWUtils.Print("Item ID "..itemID.." not found in FAS_ItemInfo.");
				end
				vendorInfoCount = vendorInfoCount + 1;
			end
			GFWUtils.Print(vendorInfoCount.." entries in FAS_VendorInfo["..faction.."].");
		end
		return;
	end

	local _, _, cmd, itemLink = string.find(msg, "(%w+) (.+)");
	if (cmd == "vendor" or cmd == "info") then
		local _, _, itemID  = string.find(itemLink, ".Hitem:(%d+):%d+:%d+:%d+.h%[[^]]+%].h");
		if (itemID == nil or itemID == "") then
			GFWUtils.Print("Usage: "..GFWUtils.Hilite("/ads info <item link>"));
			return;
		end
		itemID = tonumber(itemID);
		
		if (vendors == nil or vendors == {}) then
			GFWUtils.Print(GFWUtils.Red("AdSpace error: ")..link.."("..itemID..") is listed but has no vendors. Please report to gazmik@fizzwidget.com.");
			return;
		end

		local itemInfo = FAS_ItemInfo[itemID];
		if (itemInfo ~= nil) then
			local myFaction = UnitFactionGroup("player");
			local vendors = GFWTable.Merge(FAS_VendorInfo[myFaction][itemID], FAS_VendorInfo["Neutral"][itemID]);
			local note = itemInfo.note;
			local intro = string.format(SOLD_FOR_PRICE_BY, FAS_GetTextGSC(itemInfo.b));

			GFWUtils.Print("Found on vendors:");
			for i, aVendor in vendors do
				GFWUtils.Print(string.format(VENDOR_LOCATION_FORMAT, aVendor, FAS_VendorLocations[aVendor]));
			end
			if (note ~= nil and note ~= "") then
				GFWUtils.Print(GFWUtils.Hilite(note));
			end
			return;
		end
		
		local libramInfo = FAS_LibramInfo[itemID];
		if (libramInfo ~= nil) then
			GFWUtils.Print(RETURN_TO.." "..string.format(VENDOR_LOCATION_FORMAT, libramInfo.name, FAS_VendorLocations[libramInfo.name]));
			GFWUtils.Print(libramInfo.bonus);
			return;
		end
		
		local otherInfo = FAS_OtherItemInfo[itemID];
		if (otherInfo ~= nil) then
			GFWUtils.Print(otherInfo);
			return;
		end

		GFWUtils.Print("Nothing known about "..itemLink..".");
		return;
	end
	
	-- if we made it down here, there were args we didn't understand... time to remind the user what to do.
	FAS_ChatCommandHandler("help");

end

-- calculate the gold, silver, and copper values based the ammount of copper
function FAS_GetGSC(money)
	if (money == nil) then money = 0; end
	local g = math.floor(money / 10000);
	local s = math.floor((money - (g*10000)) / 100);
	local c = math.floor(money - (g*10000) - (s*100));
	return g,s,c;
end

-- formats money text by color for gold, silver, copper
function FAS_GetTextGSC(money)
    local GSC_GOLD="ffd100";
    local GSC_SILVER="e6e6e6";
    local GSC_COPPER="c8602c";
    local GSC_START="|cff%s%d|r";
    local GSC_PART=".|cff%s%02d|r";
    local GSC_NONE="|cffa0a0a0none|r";

	local g, s, c = FAS_GetGSC(money);

	local gsc = "";
	if (g > 0) then
		gsc = format(GSC_START, GSC_GOLD, g);     
		if (s > 0) then
			gsc = gsc..format(GSC_PART, GSC_SILVER, s);
		end
	elseif (s > 0) then
		gsc = format(GSC_START, GSC_SILVER, s);
		if (c > 0) then
			gsc = gsc..format(GSC_PART, GSC_COPPER, c);
		end
	elseif (c > 0) then
		gsc = gsc..format(GSC_START, GSC_COPPER, c);
	else
		gsc = GSC_NONE;
	end

	return gsc;
end

